import random
from django.urls import reverse
from django.utils.html import format_html, format_html_join

from django.contrib import admin

from .models import *
from .forms import PersonChangeForm

def random_color():
  r = lambda: random.randint(0,255)
  return '#{:02x}{:02x}{:02x}'.format(r(), r(), r())

@admin.register(Person)
class PersonAdmin(admin.ModelAdmin):
  list_select_related = ('parent', 'group')
  list_display = ['name', 'get_partners', 'get_parent', 'birth_date', 'death_date', 'comment', 'color', 'group']
  list_editable = ['color']#, 'group']
  list_filter = ('group', 'birth_date')
  search_fields = ['name', 'parent__name', 'partners__name']
  ordering = ('group','name')
  filter_horizontal = ('partners',)

  def get_queryset(self, request):
    qs = super().get_queryset(request)
    return qs.prefetch_related('partners')

  def get_partners(self, obj):
    return format_html_join(', ', '<a href="{}">{}</a>', [(reverse("admin:stammbaum_person_change", args=[p.id]), p) for p in obj.partners.all()])
  get_partners.short_description = "Partner"

  def get_parent(self, obj):
    if obj.parent:
      return format_html('<a href="{}">{}</a>', reverse("admin:stammbaum_person_change", args=[obj.parent.id]), obj.parent)
  get_parent.short_description = "Eltern"

  def save_related(self, request, form, formsets, change):
    super(PersonAdmin, self).save_related(request, form, formsets, change)
    partners = form.instance.partners.all()
    if not form.instance.color and len(partners) > 0:
      color = random_color()
      for partner in partners:
        if partner.color:
          color = partner.color
          break
      form.instance.color = color
      form.instance.save()
      
    for partner in partners:
      print(partner)
      partner.color = form.instance.color
      partner.save()

  #form = PersonChangeForm

@admin.register(Group)
class GroupAdmin(admin.ModelAdmin):
  list_display = ['name', 'color']
  list_editable = ['name', 'color']
  list_display_links = None
  ordering = ('name',)